@@ -45,12 +45,6 @@ int leaper_move_flags_to_betza(move_flag_t flags, char *buffer, size_t size) con
    enum sym4_t { V, S, F, B, L, R, RF4, LF4, RB4, LB4, NUM_SYM4 };
    enum sym8_t { FL, LF, FR, RF, BL, LB, BR, RB, FF, BB, LL, RR, FH, BH, LH, RH, NUM_SYM8 };
 
-   static const char *leaper_move_string[] = {
-      "*WDH",
-      "WFNC",
-      "DNAZ",
-      "HCZG"
-   };
    static struct {
       int dx, dy;
       const char *txt;
@@ -195,7 +189,7 @@ int leaper_move_flags_to_betza(move_flag_t flags, char *buffer, size_t size) con
        */
       bitboard_t<kind> kb = movegen.make_leaper_bitboard(square, 1, 1)|movegen.make_leaper_bitboard(square, 1, 0);
       if (leaper == kb) {
-         if (index2 == index)
+         if (index2 == index && is_double_leaper(flags))
             n += snprintf(buffer+n, size-n, "a");
          n += snprintf(buffer+n, size-n, "K");
          leaper.clear();
@@ -414,11 +408,11 @@ int stepper_move_flags_to_betza(move_flag_t flags, char *buffer, size_t size) co
              * Due to the way the matching works, the first iteration will always try to match the SVFBRL directions.
              * This maps (NW,NE,SE) -> fF + SE -> ffrbF.
              */
-            if (ai == 2 && strlen(sym4txt[best]) == 2 && k == 1)
+            if (ai == 1 && strlen(sym4txt[best]) == 2 && k == 1)
                k += snprintf(s+k, sizeof(s)-k, "%c", s[k-1]);
             k += snprintf(s+k, sizeof(s)-k, "%s", sym4txt[best]);
 
-            moves &= ~(sym4[k]&atombb[ai]);
+            moves &= ~(sym4[best]&atombb[ai]);
          }
 
          n += snprintf(buffer+n, size-n, "%s%s%s", s, atom[ai], countstr);
