@@ -24,6 +24,7 @@
 #include "move.h"
 #include "hashkey.h"
 #include "squares.h"
+#include "ansi.h"
 
 /* Possible rule flags */
 #define RF_FORCE_CAPTURE      0x00000001
@@ -35,7 +36,7 @@
 
 #define RF_KING_TABOO         0x00000010  /* Kings cannot face eachother along a ray */
 #define RF_KING_TRAPPED       0x00000020  /* Kings are trapped in a "palace" */
-#define RF_KING_CORNERDRIVE   0x00000040  /* Kings can drive eachother from the corners */
+#define RF_CHECK_ANY_KING     0x00000040  /* If there are multiple kings, attacking any one of them counts as check. */
 #define RF_KING_DUPLECHECK    0x00000080  /* It's check if all kings are under attack, if there is more than one. */
 
 #define RF_ALLOW_DROPS        0x00000100  /* The game allows drop moves */
@@ -50,6 +51,8 @@
 
 #define RF_SPECIAL_IS_INIT    0x00004000  /* Special moves are only initial moves */
 
+#define RF_VICTIM_SIDEEFFECT  0x00008000  /* Capture victims may have side effects */
+
 #define RF_USE_HOLDINGS       (RF_USE_DROPS | RF_USE_CAPTURE | RF_ALLOW_PICKUP)  /* Game uses holdings in some way */
 
 #define RF_USE_SHAKMATE       0x00010000  /* The checking sequence prior to mate needs particular pieces */
@@ -64,6 +67,7 @@
 #define BF_CHECK              0x0001      /* Whether the side to move is in-check or not */
 #define BF_WSHAK              0x0002      /* Whether a "shak" was given or not */
 #define BF_BSHAK              0x0004      /* Whether a "shak" was given or not */
+#define BF_NO_RETALIATE       0x0008      /* Whether retaliation is allowed or not */
 
 template <typename kind>
 struct unmake_info_t {
@@ -72,6 +76,7 @@ struct unmake_info_t {
    uint64_t board_hash;
    uint64_t hash;
    int8_t fifty_counter;
+   int8_t check_count[2];
 
    int8_t ep_victim;
 
@@ -116,6 +121,9 @@ struct board_t {
    /* Half-move clock (50-move counter) */
    int8_t fifty_counter;
 
+   /* Check count */
+   int8_t check_count[2];
+
    /* Side to move */
    side_t side_to_move;
 
@@ -131,6 +139,7 @@ struct board_t {
       board_flags &= ~BF_CHECK;
       board_flags |= uint8_t(chk);
       if (!chk) board_flags &= ~(BF_WSHAK << side_to_move);
+      if (chk)  check_count[side_to_move]++;
    }
 
    void shak() {
@@ -140,6 +149,8 @@ struct board_t {
       return (board_flags & (BF_WSHAK << side_to_move)) != 0;
    }
 
+   bool retaliate_ok() const { return !(board_flags & BF_NO_RETALIATE); }
+
    void clear()
    {
       for (side_t side = WHITE; side<NUM_SIDES; side++) bbc[side].clear();
@@ -151,6 +162,9 @@ struct board_t {
 
       memset(holdings, 0, sizeof holdings);
 
+      check_count[WHITE] = 0;
+      check_count[BLACK] = 0;
+
       hash = 0;
       board_hash = 0;
 
@@ -232,9 +246,12 @@ struct board_t {
       ui->ep = ep;
       ui->ep_victim = ep_victim;
       ui->board_flags = board_flags;
+      ui->check_count[WHITE] = check_count[WHITE];
+      ui->check_count[BLACK] = check_count[BLACK];
 #ifdef DEBUGMODE
       ui->move = move;
 #endif
+      board_flags &= ~BF_NO_RETALIATE;
 
       /* Second: resolve all pickups */
       n = get_move_pickups(move);
@@ -246,6 +263,8 @@ struct board_t {
 
          ui->pickup_piece[c] = piece_for_side(piece, side);
          clear_piece(piece, side, square);
+         if ((piece_types->piece_flags[piece] & PF_NO_RETALIATE) && side != side_to_move)
+            board_flags |= BF_NO_RETALIATE;
       }
 
       /* Third: resolve all swaps */
@@ -257,6 +276,8 @@ struct board_t {
          swap_piece[c] = get_piece(from);
          swap_side[c]  = get_side(from);
 
+         if ((piece_types->piece_flags[swap_piece[c]] & PF_NO_RETALIATE) && swap_side[c] == side_to_move)
+            board_flags &= ~BF_NO_RETALIATE;
          clear_piece(swap_piece[c], swap_side[c], from);
       }
       for (int c=0; c<n; c++)
@@ -375,6 +396,8 @@ struct board_t {
       ep = ui->ep;
       ep_victim = ui->ep_victim;
       board_flags = ui->board_flags;
+      check_count[WHITE] = ui->check_count[WHITE];
+      check_count[BLACK] = ui->check_count[BLACK];
    }
 
    void print(FILE* file = stdout, bitboard_t<kind> xmark = bitboard_t<kind>::board_empty, bitboard_t<kind> omark = bitboard_t<kind>::board_empty, bool ansi = true) const
@@ -426,17 +449,17 @@ struct board_t {
       if (file != stdout) ansi = false;
 
       for (c=virtual_ranks-1; c>=0; c--) {
-         if (ansi) fprintf(file, "%2s", rank_names[c]);
-         if (ansi) fprintf(file, "\033[1m");
+         fprintf(file, "%2s", rank_names[c]);
+         if (ansi) ansi_code("\033[1m");
          for (n=0; n<virtual_files; n++) {
             int square = n + c*virtual_files;
             int piece  = pieces[square];
             // 46/45 works for purple/cyan
             // 43/41 works for red/yellow
-            if (ansi) fprintf(file, "%s", bg_colour_string[colour[square]]);
+            if (ansi) ansi_code(bg_colour_string[colour[square]]);
             if (occupied[square]) {
                bool white = (side[square] == WHITE);
-               if (ansi) fprintf(file, "%s", white ? "\033[37m" : "\033[30m");
+               if (ansi) ansi_code(white ? "\033[37m" : "\033[30m");
                fprintf(file, "%-2s", piece_types->piece_abbreviation[piece][1-white]);
             } else {
                if (ansi)
@@ -445,17 +468,15 @@ struct board_t {
                   fprintf(file, "%c%c", colour[square] ? '+' : '.', mark[square]);
             }
          }
-         if (ansi) fprintf(file, "\033[0m");
+         if (ansi) ansi_code("\033[0m");
          if (side_to_move == BLACK && c == virtual_ranks-1) fprintf(file, "*");
          if (side_to_move == WHITE && c == 0) fprintf(file, "*");
          fprintf(file, "\n");
       }
-      if (ansi) {
-         fprintf(file, "  ");
-         for (n=0; n<virtual_files; n++)
-            fprintf(file, "%s ", file_names[n]);
-         fprintf(file, "\n");
-      }
+      fprintf(file, "  ");
+      for (n=0; n<virtual_files; n++)
+         fprintf(file, "%s ", file_names[n]);
+      fprintf(file, "\n");
 
       if (rule_flags & RF_USE_HOLDINGS) {
          for (c=0; c<NUM_SIDES; c++) {
