@@ -15,6 +15,10 @@
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+/* Integer constants are used within the evaluation itself, floating point
+ * values are only used when initialising the evaluation tables at startup.
+ */
+
 #define SAFE_MOB_WEIGHT        8       // Weight for safe mobility squares
 #define MOB_SCALE             16       // Overall mobility weight
 #define MOB_FORWARD_BLOCKED    5       // Penalty for forward mobility blocked by pieces
@@ -33,7 +37,30 @@
 
 #define PST_SPACE_MG          5        // PST value for empty squares on the own side of the board
 
-#define PASSER_RANK_SCALE     4
+#define PST_CENTRE_Q          1.0
+#define PST_CENTRE_L          0.0
+#define PST_ADVANCE_Q         0.0
+#define PST_ADVANCE_L         0.0
+
+#define PST_SCALE_PALACE      1.1      // Scale factor for PST in games with a palace
+
+#define MOBILITY_SCALE_MG   128.0      // Scale factor for mobility, middle game
+// Weight vs. STS score:
+//  96 (8001, 923)
+// 112 (8143, 932)
+// 120 (8055, 932)
+// 128 (8234, 940)
+// 130 (8110, 922)
+// 140 (8086, 926)
+// 192 (8094, 931)
+#define MOBILITY_SCALE_EG   128.0      // Scale factor for mobility, end game
+// Weight vs. STS score:
+// 120 (8114, 928)
+// 128 (8234, 940)
+// 140 (8007, 916)
+
+#define PASSER_RANK_BASE      4
+#define PASSER_RANK_SCALE   128
 
 #define PAWN_ADVANCE_Q_EG    98.0      // Bonus for advancing pawns in the end game, quadratic term
 #define PAWN_ADVANCE_L_EG     0.0      // Bonus for advancing pawns in the end game, linear term
@@ -54,6 +81,9 @@
 #define SLIDER_OPENFILE_MG   10        // Slider bonus on open file, middle game
 #define SLIDER_OPENFILE_EG   15        // Slider bonus on open file, end game
 
+#define HOPPER_KINGFILE_MG   50        // Hopper on same file as enemy king, middle game
+#define HOPPER_KINGFILE_EG    0        // Hopper on same file as enemy king, middle game
+
 #define DEF_PROTECT           7        // Bonus for defensive pieces that protect eachother
 #define DEF_SHIELD_FILE      10        // Bonus for defensive pieces on king file
 
