@@ -141,6 +141,13 @@ eval_t game_template_t<kind>::static_evaluation(side_t side_to_move, int /* alph
    }
 #endif
 
+   /* Randomised evaluation */
+   if (level == LEVEL_BEAL) {
+      ev = genrandui() & 0x7ff;
+      if (side_to_move == BLACK) ev = -ev;
+      goto exit;
+   }
+
    calculate_pawn_structure(&ps);
 
    for (side_t side = WHITE; side<NUM_SIDES; side++) {
@@ -188,12 +195,12 @@ eval_t game_template_t<kind>::static_evaluation(side_t side_to_move, int /* alph
                   phase += pt.phase_weight[piece] * board.holdings[piece][side];
                   pst = 0;
                } else {
-                  phase += pt.phase_weight[piece] * board.holdings[piece][side] * scale;
+                  phase += (int)(pt.phase_weight[piece] * board.holdings[piece][side] * scale);
                   pst = 0;
                }
 
                mat += pt.eval_value[piece] * board.holdings[piece][side] * scale;
-               psq += pst * board.holdings[piece][side] * scale;
+               psq += (int)(pst * board.holdings[piece][side] * scale);
             }
          } else {
             mat += pt.eval_value[piece] * board.holdings[piece][side];
@@ -220,10 +227,9 @@ eval_t game_template_t<kind>::static_evaluation(side_t side_to_move, int /* alph
             if (pt.defensive_pieces & (1<<piece)) {
                def[side] += pt.eval_value[piece];
                defence.set(square);
-               mat += pt.eval_value[piece] / 2;
                num_def[side]++;
-            } else
-               mat += pt.eval_value[piece];
+            }
+            mat += pt.eval_value[piece];
 
             psq   += pt.eval_pst[piece][psq_map[side][square]];
             phase += pt.phase_weight[piece];
@@ -301,10 +307,12 @@ eval_t game_template_t<kind>::static_evaluation(side_t side_to_move, int /* alph
    }
 
    /* Take into account defensive material */
-   if (mat.mg < 0) mat.mg = std::min(0, mat.mg + def[WHITE].mg/2);
-   if (mat.eg < 0) mat.eg = std::min(0, mat.eg + def[WHITE].eg/2);
-   if (mat.mg > 0) mat.mg = std::max(0, mat.mg - def[BLACK].mg/2);
-   if (mat.eg > 0) mat.eg = std::max(0, mat.eg - def[BLACK].eg/2);
+   // FIXME: doing it this way allows the program to drop a piece, which it
+   // thinks is "fine" because it still has its defensive pieces.
+   //if (mat.mg < 0) mat.mg = std::min(0, mat.mg + def[WHITE].mg/2);
+   //if (mat.eg < 0) mat.eg = std::min(0, mat.eg + def[WHITE].eg/2);
+   //if (mat.mg > 0) mat.mg = std::max(0, mat.mg - def[BLACK].mg/2);
+   //if (mat.eg > 0) mat.eg = std::max(0, mat.eg - def[BLACK].eg/2);
 
    /* Gather data for shelter and assess winning chances. */
    for (side_t side = WHITE; side<NUM_SIDES; side++) {
@@ -322,16 +330,15 @@ eval_t game_template_t<kind>::static_evaluation(side_t side_to_move, int /* alph
          score = ps.shelter_score[side][f];  /* 0-8 */
          shelter[side] = 4*score;            /* 0-32 */
 
-         if ((board.init & movegen.castle_mask[SHORT][side] & board.bbc[side]) == movegen.castle_mask[SHORT][side]) {
-            int square = movegen.castle_king_dest[SHORT][side];
-            int f = unpack_file(square);
-            cscore = std::max(cscore, ps.shelter_score[side][f]);
-         }
-
-         if ((board.init & movegen.castle_mask[LONG][side] & board.bbc[side]) == movegen.castle_mask[LONG][side]) {
-            int square = movegen.castle_king_dest[LONG][side];
-            int f = unpack_file(square);
-            cscore = std::max(cscore, ps.shelter_score[side][f]);
+         for (int c = SHORT; c<NUM_CASTLE_MOVES; c++)
+         if ((board.init & movegen.castle_mask[c][side] & board.bbc[side]) == movegen.castle_mask[c][side]) {
+            bitboard_t<kind> bb = movegen.castle_king_dest[c][side];
+            while (!bb.is_empty()) {
+               int square = bb.bitscan();
+               int f = unpack_file(square);
+               bb.reset(square);
+               cscore = std::max(cscore, ps.shelter_score[side][f]);
+            }
          }
 
          //printf("Shelter: %d %d\n", score, cscore);
@@ -497,7 +504,7 @@ eval_t game_template_t<kind>::static_evaluation(side_t side_to_move, int /* alph
             if ( board.rule_flags & RF_USE_HOLDINGS ) {
                for (int piece = 0; piece<pt.num_piece_types; piece++) {
                   if (board.holdings[piece][side]) {
-                     ka_weight += std::max(1, pt.phase_weight[piece]);
+                     ka_weight += std::max(1, pt.king_safety_weight[piece]);
                      ka_count++;
                   }
                }
@@ -510,7 +517,7 @@ eval_t game_template_t<kind>::static_evaluation(side_t side_to_move, int /* alph
 
             /* King attack pattern */
             if ((!(attack[square] & ~less_attacks[oside][piece] & king_zone).is_empty())) {
-               ka_weight += std::max(1, pt.phase_weight[piece]);
+               ka_weight += std::max(1, pt.king_safety_weight[piece]);
                ka_count++;
             }
 
@@ -546,7 +553,7 @@ eval_t game_template_t<kind>::static_evaluation(side_t side_to_move, int /* alph
                   int rank = unpack_rank(square);
                   if (side == BLACK) rank = (bitboard_t<kind>::board_ranks - 1) - rank;
 
-                  eval_t scale = PASSER_RANK_SCALE;
+                  eval_t scale = PASSER_RANK_BASE;
 
                   bitboard_t<kind> fs = pt.front_span[side][square];
                   if ((fs & occ).is_empty()) scale++;
@@ -554,6 +561,8 @@ eval_t game_template_t<kind>::static_evaluation(side_t side_to_move, int /* alph
                   if (all_attacks[side].test(square)) scale++;
                   if ((fs & all_attacks[side]) == fs) scale++;
                   if ((fs & all_attacks[oside]).is_empty()) scale++;
+                  scale *= PASSER_RANK_SCALE;
+                  scale /= 128;
 
                   pss.eg += scale*rank*rank;
                }
@@ -634,6 +643,14 @@ eval_t game_template_t<kind>::static_evaluation(side_t side_to_move, int /* alph
                   psq += DEF_SHIELD_FILE;
             }
 
+            /* Hoppers on same file as king (unblocked) */
+            if (is_hopper(pt.piece_capture_flags[piece]) &&
+               unpack_file(square) == unpack_file(king[oside]) && 
+               bitboard_t<kind>::board_between[square][king[oside]].is_empty()) {
+               kss.mg += HOPPER_KINGFILE_MG;
+               kss.eg += HOPPER_KINGFILE_EG;
+            }
+
             si++;
          }
 
@@ -680,7 +697,6 @@ eval_t game_template_t<kind>::static_evaluation(side_t side_to_move, int /* alph
    }
 
 
-done:
    if (print) {
       printf("Component:       MG:    EG:\n");
       printf("Material:        % 4d   % 4d\n", mat.mg, mat.eg);
@@ -698,6 +714,9 @@ done:
    /* Interpolate evaluation score. */
    ev = (mat + psq + mob + pss + kss).interpolate(phase, pt.phase_scale);
 
+   if (check_limit)
+      ev += -(board.check_count[WHITE] - board.check_count[BLACK])*check_score / (20*check_limit);
+
    /* Adjust the score: if the side that is nominally ahead can't win, drop the score to 0(ish) */
    phase = std::min(phase, pt.phase_scale);
    if (!can_win[WHITE] && !can_win[BLACK]) ev = 0;
@@ -730,8 +749,18 @@ done:
 exit:
 
    /* Tapered evaluation when we're about to hit the 50 move counter */
-   if (board.fifty_counter > 80)
-      ev = ev * (101 - board.fifty_counter) / 20;
+   if (fifty_scale_limit && board.fifty_counter > fifty_scale_limit)
+      ev = ev * (fifty_limit - board.fifty_counter) / (fifty_limit - fifty_scale_limit);
+
+   /* Add a pseudo-random contribution to the opening moves */
+   if (random_ok && random_amplitude && (start_move_count + moves_played) < random_ply_count) {
+      unsigned int rand = (unsigned int)(board.hash ^ random_key);
+      int amp = int(random_amplitude * (random_ply_count - start_move_count - moves_played) / random_ply_count);
+      int r = ((int)(rand & 0xff) - 0x7f) * amp / 0x7f;
+      if (print)
+         printf("Random factor:   % 4d\n", r);
+      ev += r;
+   }
 
    return (side_to_move == WHITE) ? ev : -ev;
 }
