@@ -13,11 +13,36 @@ void record_castle_state(char state, bitboard_t<kind> *castle_init)
    side_t side = BLACK;
    if (isupper(state)) side = WHITE;
 
+   /* Fischer-random style castling. This identifies the file of the piece with which
+    * we can castle. We can find the file of the king by looking at the royal bitboard, and we
+    * know the king destination after castling from the rule description. From this, we can work
+    * out the required bitmasks.
+    */
+   int king_from = -1;
+   for (int n = 0; n<pt.num_piece_types; n++) {
+      if (pt.piece_flags[n] & PF_CASTLE) {
+         king_from = (board.bbc[side] & board.bbp[n]).bitscan();
+         break;
+      }
+   }
+   if (king_from < 0) {
+      /* No castling, but we should still mark virgin pieces */
+      bitboard_t<kind> side_mask = bitboard_t<kind>::board_rank[(side == WHITE) ? 0 : board_ranks-1];
+      char file_char = (side == WHITE) ? 'A' : 'a';
+      int file = state - file_char;
+      if (file >= board_files || file < 0) return;
+
+      *castle_init |= bitboard_t<kind>::board_file[file] & side_mask;
+
+      return;
+   }
+   int king_rank = unpack_rank(king_from);
+
    char file_char = (side == WHITE) ? 'A' : 'a';
 
    if (state-file_char < board_files) {
       rook_file = state - file_char;
-      rook_from = bitboard_t<kind>::pack_rank_file( (side == WHITE) ? 0 : (board_ranks-1), rook_file);
+      rook_from = bitboard_t<kind>::pack_rank_file(king_rank, rook_file);
 
       castle_init->set(rook_from);
 
@@ -31,8 +56,9 @@ void record_castle_state(char state, bitboard_t<kind> *castle_init)
           */
          state = '-';
          for (int n = board_files - 1; n>=0; n--) {
-            if (board.get_piece(n) == pt.castle_piece[side][SHORT]) {
-               rook_from = n;
+            int sqr = bitboard_t<kind>::pack_rank_file(king_rank, n);
+            if (board.get_piece(sqr) == pt.castle_piece[side][SHORT]) {
+               rook_from = sqr;
                state = file_char + n;
                break;
             }
@@ -48,8 +74,9 @@ void record_castle_state(char state, bitboard_t<kind> *castle_init)
           */
          state = '-';
          for (int n = 0; n<board_files; n++) {
-            if (board.get_piece(n) == pt.castle_piece[side][LONG]) {
-               rook_from = n;
+            int sqr = bitboard_t<kind>::pack_rank_file(king_rank, n);
+            if (board.get_piece(sqr) == pt.castle_piece[side][LONG]) {
+               rook_from = sqr;
                state = file_char + n;
                break;
             }
@@ -65,9 +92,9 @@ void record_castle_state(char state, bitboard_t<kind> *castle_init)
           */
          state = '-';
          for (int n = board_files - 1; n>=0; n--) {
-            int s = board_files * (board_ranks-1);
-            if (board.get_piece(s + n) == pt.castle_piece[side][SHORT]) {
-               rook_from = s+n;
+            int sqr = bitboard_t<kind>::pack_rank_file(king_rank, n);
+            if (board.get_piece(sqr) == pt.castle_piece[side][SHORT]) {
+               rook_from = sqr;
                state = file_char + n;
                break;
             }
@@ -83,9 +110,9 @@ void record_castle_state(char state, bitboard_t<kind> *castle_init)
           */
          state = '-';
          for (int n = 0; n<board_files; n++) {
-            int s = board_files * (board_ranks-1);
-            if (board.get_piece(s + n) == pt.castle_piece[side][LONG]) {
-               rook_from = s+n;
+            int sqr = bitboard_t<kind>::pack_rank_file(king_rank, n);
+            if (board.get_piece(sqr) == pt.castle_piece[side][LONG]) {
+               rook_from = sqr;
                state = file_char + n;
                break;
             }
@@ -98,29 +125,20 @@ void record_castle_state(char state, bitboard_t<kind> *castle_init)
 
    if (rook_from < 0) return;
 
-   /* Fischer-random style castling. This identifies the file of the piece with which
-    * we can castle. We can find the file of the king by looking at the royal bitboard, and we
-    * know the king destination after castling from the rule description. From this, we can work
-    * out the required bitmasks.
-    */
-   int king_from = -1, king_to;
-
-   for (int n = 0; n<pt.num_piece_types; n++) {
-      if (pt.piece_flags[n] & PF_CASTLE) {
-         king_from = (board.bbc[side] & board.bbp[n]).bitscan();
-         break;
-      }
-   }
-   if (king_from < 0) return;
    int castle_side = LONG;
    if (unpack_file(rook_from) >= unpack_file(king_from)) castle_side = SHORT;
 
-   king_to = movegen.castle_king_dest[castle_side][side];
+   bitboard_t<kind> king_dest = movegen.castle_king_dest[castle_side][side];
+   movegen.clear_castle_rule(castle_side, side);
+   while (!king_dest.is_empty()) {
+      int king_to = king_dest.bitscan();
+      king_dest.reset(king_to);
 
-   //printf("%d (%s %s) (%s)\n", side, square_names[king_from], square_names[king_to], square_names[rook_from]);
-   movegen.deduce_castle_flags(side, king_from, king_to, rook_from);
+      //printf("%d (%s %s) (%s)\n", side, square_names[king_from], square_names[king_to], square_names[rook_from]);
+      movegen.deduce_castle_flags(side, king_from, king_to, rook_from);
 
-   castle_init->set(king_from);
+      castle_init->set(king_from);
+   }
 
    return;
 }
@@ -175,7 +193,7 @@ void setup_fen_position(const char *str, bool skip_castle = false)
                if (strstr(s, pt.piece_abbreviation[n][BLACK]) == s) match_side = BLACK;
 
                if (match_side != NONE) {
-                  int len = strlen(pt.piece_abbreviation[n][match_side]);
+                  int len = (int)strlen(pt.piece_abbreviation[n][match_side]);
                   if (len > best_len) {
                      best_len = len;
                      side = match_side;
@@ -185,6 +203,7 @@ void setup_fen_position(const char *str, bool skip_castle = false)
             }
             if (piece == -1) {
                if (error_output) error_output("Error: unknown piece type '%c' (bad FEN %s)\n", *s, str);
+               if (xboard_output) xboard_output("tellusererror unknown piece type '%c' (bad FEN %s)\n\n", *s, str);
                return;
             }
             assert(side != NONE);
@@ -268,7 +287,7 @@ void setup_fen_position(const char *str, bool skip_castle = false)
          /* En-passant move-to square */
          board.ep.set(bitboard_t<kind>::pack_rank_file(rank, file));
          /* En-passant capture square, this may be encoded in the FEN */
-         if (!isspace(*s)) {
+         if (*s && !isspace(*s)) {
             int file = *s - 'a';
             int rank = s[1] - '1';
             s+=2;
