@@ -6,7 +6,7 @@ void update_drop_history(move_t move, int score)
 #ifdef USE_HISTORY_HEURISTIC
    int history_score;
 
-   int piece = get_move_piece(move);
+   int piece = board.get_piece(get_move_from(move));
    int side  = board.side_to_move;
    int to    = get_move_to(move);
 
@@ -39,7 +39,7 @@ void update_history(move_t move, int score)
       return;
    }
 
-   int piece = get_move_piece(move);
+   int piece = board.get_piece(get_move_from(move));
    int side  = board.side_to_move;
    int to    = get_move_to(move);
 
@@ -62,7 +62,7 @@ void update_history(move_t move, int score)
 int get_move_history_score(const move_t move) const
 {
 #ifdef USE_HISTORY_HEURISTIC
-   int piece = get_move_piece(move);
+   int piece = board.get_piece(get_move_from(move));
    int side  = board.side_to_move;
    int to    = get_move_to(move);
    if (is_drop_move(move))
