@@ -77,7 +77,7 @@ typedef uint64_t move_t;
 #define MOVE_PIECE_MASK       ((1<<MOVE_PIECE_BITS)-1)
 #define MOVE_DROP_MASK        ((1<<MOVE_DROP_SIZE)-1)
 #define MOVE_PICKUP_MASK      ((1<<MOVE_PICKUP_SIZE)-1)
-#define MOVE_HOLDING_MASK     ((1<<(MOVE_PIECE_BITS + MOVE_SIDE_BITS + 2))-1)
+#define MOVE_HOLDING_MASK     ((1<<(MOVE_PIECE_BITS + MOVE_SIDE_BITS + 1))-1)
 #define MOVE_SWAP_MASK        ((1<<MOVE_SWAP_SIZE)-1)
 #define MOVE_SIDE_MASK        0x0001
 
@@ -231,7 +231,7 @@ static inline int decode_pickup_square(int pickup)
 /* The code for holdings is almost the same as for drops */
 static inline int decode_holding_count(int hold)
 {
-   return decode_drop_square(hold) - 2;
+   return 2*decode_drop_square(hold) - 1;
 }
 #define decode_holding_piece decode_drop_piece
 #define decode_holding_side decode_drop_side
@@ -265,10 +265,9 @@ static inline move_t encode_normal_move(uint8_t piece, uint8_t from, uint8_t to)
 }
 
 /* Encode an en-passant capture, where the pickup square is different from the destination square */
-static inline move_t encode_en_passant_capture(uint8_t piece, uint8_t from, uint8_t to, uint8_t ptaken, uint8_t ep)
+static inline move_t encode_en_passant_capture(uint8_t piece, uint8_t from, uint8_t to, uint8_t ep)
 {
    int swap_shift = MOVE_PICKUP_SIZE;
-   (void)ptaken;
 
    return encode_number_pickups(1) |
           encode_number_swaps(1)   |
@@ -278,17 +277,15 @@ static inline move_t encode_en_passant_capture(uint8_t piece, uint8_t from, uint
 }
 
 /* Encode a normal capture */
-static inline move_t encode_normal_capture(uint8_t piece, uint8_t from, uint8_t to, uint8_t ptaken)
+static inline move_t encode_normal_capture(uint8_t piece, uint8_t from, uint8_t to)
 {
-   return encode_en_passant_capture(piece, from, to, ptaken, to);
+   return encode_en_passant_capture(piece, from, to, to);
 }
 
 /* Encode an en-passant capture, where the pickup square is different from the destination square */
-static inline move_t encode_double_capture(uint8_t piece, uint8_t from, uint8_t to, uint8_t ptaken, uint8_t to2, uint8_t ptaken2)
+static inline move_t encode_double_capture(uint8_t piece, uint8_t from, uint8_t to, uint8_t to2)
 {
    int swap_shift = 2*MOVE_PICKUP_SIZE;
-   (void)ptaken;
-   (void)ptaken2;
 
    return encode_number_pickups(2) |
           encode_number_swaps(1)   |
@@ -309,9 +306,8 @@ static inline move_t encode_normal_promotion(uint8_t piece, uint8_t from, uint8_
 }
 
 /* Encode a capture promotion */
-static inline move_t encode_capture_promotion(uint8_t piece, uint8_t from, uint8_t to, uint8_t ptaken, uint8_t tpiece)
+static inline move_t encode_capture_promotion(uint8_t piece, uint8_t from, uint8_t to, uint8_t tpiece)
 {
-   (void)ptaken;
    return encode_number_pickups(2) |
           encode_number_drops(1) |
           encode_move_piece(piece) |
@@ -350,7 +346,7 @@ static inline move_t add_move_store(move_t move, uint8_t piece, int8_t count)
    assert(count > -2);
    assert((get_move_pickups(move) + get_move_drops(move)) < 5);
 
-   count += 2;
+   count  = (count + 1) / 2;
    shift = MOVE_SLOT1 +
            MOVE_DROP_SIZE * get_move_drops(move) +
            MOVE_SWAP_SIZE * get_move_swaps(move) +
