@@ -14,7 +14,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
-int add_piece_type(move_flag_t move_flags, move_flag_t capture_flags, uint16_t piece_flags,
+int add_piece_type(move_flag_t move_flags, move_flag_t capture_flags, piece_flag_t piece_flags,
       const bitboard_t<kind> promotion_zone[2], const char *promotion_string,
       const char *name, const char *symbol, const char *notation, int value = 0)
 {
@@ -31,29 +31,36 @@ int add_piece_type(move_flag_t move_flags, move_flag_t capture_flags, uint16_t p
    free(pt.piece_name[n]);
    free(pt.piece_abbreviation[n][WHITE]);
    free(pt.piece_notation[n]);
-   free(pt.promotion_string[n]);
    free(pt.demotion_string[n]);
+   memset(pt.promotion[n], 0, sizeof pt.promotion[n]);
 
    pt.piece_move_flags[n]     = move_flags;
    pt.piece_capture_flags[n]  = capture_flags;
    pt.piece_flags[n]          = piece_flags;
    pt.special_zone[WHITE][n]  = bitboard_t<kind>::board_empty;
    pt.special_zone[BLACK][n]  = bitboard_t<kind>::board_empty;
-   pt.promotion_zone[WHITE][n] = promotion_zone[WHITE];
-   pt.promotion_zone[BLACK][n] = promotion_zone[BLACK];
+   pt.promotion[n][0].zone[WHITE] = promotion_zone[WHITE];
+   pt.promotion[n][0].zone[BLACK] = promotion_zone[BLACK];
+   pt.promotion[n][0].string = strdup(promotion_string);
+   pt.promotion[n][0].choice = 0;
+   pt.promotion_zone[WHITE][n].clear();
+   pt.promotion_zone[BLACK][n].clear();
    pt.optional_promotion_zone[WHITE][n].clear();
    pt.optional_promotion_zone[BLACK][n].clear();
    pt.prison[WHITE][n]        = bitboard_t<kind>::board_all;
    pt.prison[BLACK][n]        = bitboard_t<kind>::board_all;
+   pt.block[WHITE][n]         = bitboard_t<kind>::board_empty;
+   pt.block[BLACK][n]         = bitboard_t<kind>::board_empty;
    pt.drop_zone[WHITE][n]     = bitboard_t<kind>::board_all;
    pt.drop_zone[BLACK][n]     = bitboard_t<kind>::board_all;
    pt.piece_name[n]           = strdup(name);
    pt.piece_notation[n]       = strdup(notation);
-   pt.promotion_string[n]     = strdup(promotion_string);
    pt.demotion_string[n]      = NULL;
+   pt.allowed_victim[n]       = NULL;
    pt.piece_value[n]          = value;
    pt.demotion[n]             = n;
    pt.piece_promotion_choice[n] = 0;
+   pt.piece_allowed_victims[n] = ~0;
    pt.pzset[n]                = false;
    pt.piece_drop_file_maximum[n] = 1;
    if (piece_flags & PF_ROYAL) {
@@ -119,6 +126,24 @@ void set_maximum_number_of_kings(side_t side, int count)
          pt.piece_maximum[n][side] = count;
 }
 
+bool add_initial_move(const char *symbol, move_flag_t move_flags)
+{
+   int n;
+
+   if (!symbol)
+      return false;
+
+   for (n=0; n<pt.num_piece_types; n++) {
+      if (strcmp(pt.piece_abbreviation[n][WHITE], symbol) == 0 || strcmp(pt.piece_abbreviation[n][BLACK], symbol) == 0) {
+         pt.piece_initial_move_flags[n] = move_flags;
+         return true;
+      }
+   }
+
+   return false;
+}
+
+
 void set_maximum_number_of_pieces(const char *symbol, side_t side, int count)
 {
    for (int n=0; n<pt.num_piece_types; n++) {
@@ -173,11 +198,13 @@ void identify_promotion_options()
 {
    char buffer[3];
    for (int n = 0; n<pt.num_piece_types; n++) {
-      uint16_t promotion_choice = 0;
+      pt.piece_promotion_choice[n] = 0;
 
       /* Store allowed pieces for promotion in a bit field */
-      if (pt.promotion_string[n] && pt.promotion_string[n][0]) {
-         const char *s = pt.promotion_string[n];
+      for (int k=0; k<MAX_PZ; k++) {
+      piece_bit_t promotion_choice = 0;
+      if (pt.promotion[n][k].string && pt.promotion[n][k].string[0]) {
+         const char *s = pt.promotion[n][k].string;
          while (*s) {
             if (*s == '+') {
                buffer[0] = '+';
@@ -211,7 +238,16 @@ void identify_promotion_options()
          }
       }
 
-      pt.piece_promotion_choice[n] = promotion_choice;
+      pt.piece_promotion_choice[n] |= promotion_choice;
+      pt.promotion[n][k].choice = promotion_choice;
+      }
+   }
+
+   for (side_t side=WHITE; side<=BLACK; side++)
+   for (int n = 0; n<pt.num_piece_types; n++) {
+      pt.promotion_zone[side][n].clear();
+      for (int k=0; k<MAX_PZ; k++)
+         pt.promotion_zone[side][n] |= pt.promotion[n][k].zone[side];
    }
 
    for (int n = 0; n<pt.num_piece_types; n++) {
@@ -225,4 +261,24 @@ void identify_promotion_options()
          }
       }
    }
+
+   for (int n = 0; n<pt.num_piece_types; n++) {
+      const char *s = pt.allowed_victim[n];
+      if (s == NULL) continue;
+
+      pt.piece_allowed_victims[n] = 0;
+      while (*s) {
+         const char *p = s;
+         while (*p && *p != ',') p++;
+         while (*p == ' ') p++;
+         for (int c=0; c<pt.num_piece_types; c++) {
+            if (strcmp(pt.piece_abbreviation[c][WHITE], s) == 0 ||
+                strcmp(pt.piece_abbreviation[c][BLACK], s) == 0) {
+               pt.piece_allowed_victims[n] |= 1<<c;
+            }
+         }
+         s = p+1;
+         if (!*p) break;
+      }
+   }
 }
