@@ -25,30 +25,45 @@
 #include "eval_types.h"
 
 /* Piece flags */
-#define PF_ROYAL        0x0001   /* Piece is royal */
-#define PF_CASTLE       0x0002   /* Piece may castle */
-#define PF_SET_EP       0x0004   /* Piece sets "en-passant" square when doing a special move */
-#define PF_TAKE_EP      0x0008   /* Piece may capture en-passant */
+#define PF_ROYAL        0x00000001  /* Piece is royal */
+#define PF_CASTLE       0x00000002  /* Piece may castle */
+#define PF_SET_EP       0x00000004  /* Piece sets "en-passant" square when doing a special move */
+#define PF_TAKE_EP      0x00000008  /* Piece may capture en-passant */
 
-#define PF_DROPNOCHECK  0x0010   /* Piece may not be dropped if it gives check */
-#define PF_DROPNOMATE   0x0020   /* Piece may not be dropped if it gives mate */
-#define PF_DROPONEFILE  0x0040   /* Piece may not be dropped if another piece of this type is on the same file */
-#define PF_DROPDEAD     0x0080   /* Piece may be dropped where it has no moves */
+#define PF_DROPNOCHECK  0x00000010  /* Piece may not be dropped if it gives check */
+#define PF_DROPNOMATE   0x00000020  /* Piece may not be dropped if it gives mate */
+#define PF_DROPONEFILE  0x00000040  /* Piece may not be dropped if another piece of this type is on the same file */
+#define PF_DROPDEAD     0x00000080  /* Piece may be dropped where it has no moves */
 
-#define PF_CANTMATE     0x0100   /* This piece, by itself, cannot mate */
-#define PF_COLOURBOUND  0x0200   /* This piece is colour-bound */
-#define PF_PAIRBONUS    0x0400   /* Piece gets a pair bonus for similar pieces on opposite colour */
-#define PF_NORET        0x0800   /* Piece cannot return to its original position */
+#define PF_CANTMATE     0x00000100  /* This piece, by itself, cannot mate */
+#define PF_COLOURBOUND  0x00000200  /* This piece is colour-bound */
+#define PF_PAIRBONUS    0x00000400  /* Piece gets a pair bonus for similar pieces on opposite colour */
+#define PF_NORET        0x00000800  /* Piece cannot return to its original position */
 
-#define PF_NOMATE       0x1000   /* Piece may not give mate (it may give check) */
-#define PF_SHAK         0x2000
-#define PF_CAPTUREFLAG  0x4000   /* This piece is allowed to capture the flag */
+#define PF_NOMATE       0x00001000  /* Piece may not give mate (it may give check) */
+#define PF_SHAK         0x00002000
+#define PF_CAPTUREFLAG  0x00004000  /* This piece is allowed to capture the flag */
+#define PF_ASSIMILATE   0x00008000  /* This piece assimilates a cpaturing piece */
+
+#define PF_NO_RETALIATE 0x00010000  /* A piece with this property may not be recaptured. */
+#define PF_ENDANGERED   0x00020000  /* A piece with this property may only capture unprotected pieces with this property */
+#define PF_IRON         0x00040000  /* A piece with this property may not be captured */
+
+typedef uint32_t piece_flag_t;
 
 struct value_comparator_t {
    int sort_value[256];
    bool operator()(int a, int b) const { return sort_value[a] < sort_value[b]; }
 };
 
+#define MAX_PZ    16
+template<typename kind>
+struct promotion_zone_t {
+   bitboard_t<kind>  zone[NUM_SIDES];
+   const char *      string;
+   piece_bit_t       choice;
+};
+
 template<typename kind>
 struct piece_description_t {
    /* Define the properties of all piece types.
@@ -70,34 +85,39 @@ struct piece_description_t {
     *    shield_weight        (8 bit)
     *    mobility_score       (8 bit)
     */
+   promotion_zone_t<kind> promotion[MAX_PIECE_TYPES][MAX_PZ];
    bitboard_t<kind>  promotion_zone[NUM_SIDES][MAX_PIECE_TYPES];
    bitboard_t<kind>  optional_promotion_zone[NUM_SIDES][MAX_PIECE_TYPES];
    bitboard_t<kind>  special_zone[NUM_SIDES][MAX_PIECE_TYPES];
+   bitboard_t<kind>  prison[NUM_SIDES][MAX_PIECE_TYPES];
+   bitboard_t<kind>  block[NUM_SIDES][MAX_PIECE_TYPES];
+   bitboard_t<kind>  drop_zone[NUM_SIDES][MAX_PIECE_TYPES];
+
+   int          num_piece_types;
+   int          pawn_steps[NUM_SIDES];  /* The number of moves a pawn has, a factor in pawn mobility and passer eval */
+   int8_t       pawn_index[NUM_SIDES];  /* The piece type that will be evaluated as a pawn */
+   int8_t       castle_piece[NUM_SIDES][NUM_CASTLE_MOVES];/* The piece type that will castle with the king, nominally "rook" */
+
+   move_flag_t  piece_move_flags[MAX_PIECE_TYPES];
+   move_flag_t  piece_capture_flags[MAX_PIECE_TYPES];
+   move_flag_t  piece_special_move_flags[MAX_PIECE_TYPES];
+   move_flag_t  piece_initial_move_flags[MAX_PIECE_TYPES];
+   piece_bit_t  piece_promotion_choice[MAX_PIECE_TYPES];
+   piece_bit_t  piece_allowed_victims[MAX_PIECE_TYPES];
+   piece_flag_t piece_flags[MAX_PIECE_TYPES];
+   int8_t       demotion[MAX_PIECE_TYPES];
+   uint8_t      piece_maximum[MAX_PIECE_TYPES][NUM_SIDES];
+   uint8_t      piece_drop_file_maximum[MAX_PIECE_TYPES];
+   bool         pieces_can_win[MAX_PIECE_TYPES][MAX_PIECE_TYPES];
+
+   int          val_perm[MAX_PIECE_TYPES];
+   int16_t      piece_value[MAX_PIECE_TYPES];
+   int16_t      see_piece_value[MAX_PIECE_TYPES];
+   int16_t      piece_promotion_value[MAX_PIECE_TYPES];
+
    bitboard_t<kind>  passer_mask[NUM_SIDES][8*sizeof(kind)];
    bitboard_t<kind>  weak_mask[NUM_SIDES][8*sizeof(kind)];
    bitboard_t<kind>  front_span[NUM_SIDES][8*sizeof(kind)];
-   bitboard_t<kind>  prison[NUM_SIDES][MAX_PIECE_TYPES];
-   bitboard_t<kind>  drop_zone[NUM_SIDES][MAX_PIECE_TYPES];
-
-   int         num_piece_types;
-   int         pawn_steps[NUM_SIDES];  /* The number of moves a pawn has, a factor in pawn mobility and passer eval */
-   int8_t      pawn_index[NUM_SIDES];  /* The piece type that will be evaluated as a pawn */
-   int8_t      castle_piece[NUM_SIDES][NUM_CASTLE_MOVES];/* The piece type that will castle with the king, nominally "rook" */
-
-   move_flag_t piece_move_flags[MAX_PIECE_TYPES];
-   move_flag_t piece_capture_flags[MAX_PIECE_TYPES];
-   move_flag_t piece_special_move_flags[MAX_PIECE_TYPES];
-   piece_bit_t piece_promotion_choice[MAX_PIECE_TYPES];
-   uint16_t    piece_flags[MAX_PIECE_TYPES];
-   int8_t      demotion[MAX_PIECE_TYPES];
-   uint8_t     piece_maximum[MAX_PIECE_TYPES][NUM_SIDES];
-   uint8_t     piece_drop_file_maximum[MAX_PIECE_TYPES];
-   bool        pieces_can_win[MAX_PIECE_TYPES][MAX_PIECE_TYPES];
-
-   int         val_perm[MAX_PIECE_TYPES];
-   int16_t     piece_value[MAX_PIECE_TYPES];
-   int16_t     see_piece_value[MAX_PIECE_TYPES];
-   int16_t     piece_promotion_value[MAX_PIECE_TYPES];
 
    /* Piece classifications */
    piece_bit_t royal_pieces;
@@ -127,8 +147,8 @@ struct piece_description_t {
    char *      piece_name[MAX_PIECE_TYPES];
    char *      piece_abbreviation[MAX_PIECE_TYPES][NUM_SIDES];
    char *      piece_notation[MAX_PIECE_TYPES];
-   char *      promotion_string[MAX_PIECE_TYPES];
    char *      demotion_string[MAX_PIECE_TYPES];
+   char *      allowed_victim[MAX_PIECE_TYPES];
 
    bool        pzset[MAX_PIECE_TYPES];
 
@@ -211,6 +231,24 @@ struct piece_description_t {
          if (piece_capture_flags[n] & MF_STEPPER)
             printf("   Stepper %08x (capture)\n", piece_capture_flags[n] & MF_STEPPER);
 
+         if (piece_special_move_flags[n] & MF_SLIDER)
+            printf("   Slider  %08x (special)\n", piece_special_move_flags[n] & MF_SLIDER);
+         if (piece_special_move_flags[n] & MF_HOPPER)
+            printf("   Hopper  %08x (special)\n", piece_special_move_flags[n] & MF_HOPPER);
+         if (piece_special_move_flags[n] & MF_IS_LEAPER)
+            printf("   Leaper  %08x (special)\n", piece_special_move_flags[n] & 0xffff0000);
+         if (piece_special_move_flags[n] & MF_STEPPER)
+            printf("   Stepper %08x (special)\n", piece_special_move_flags[n] & MF_STEPPER);
+
+         if (piece_initial_move_flags[n] & MF_SLIDER)
+            printf("   Slider  %08x (initial)\n", piece_initial_move_flags[n] & MF_SLIDER);
+         if (piece_initial_move_flags[n] & MF_HOPPER)
+            printf("   Hopper  %08x (initial)\n", piece_initial_move_flags[n] & MF_HOPPER);
+         if (piece_initial_move_flags[n] & MF_IS_LEAPER)
+            printf("   Leaper  %08x (initial)\n", piece_initial_move_flags[n] & 0xffff0000);
+         if (piece_initial_move_flags[n] & MF_STEPPER)
+            printf("   Stepper %08x (initial)\n", piece_initial_move_flags[n] & MF_STEPPER);
+
          printf("   Move statistics: %d--%d <%d>\n", min_moves[n], max_moves[n], avg_moves[n]);
 
          if (piece_flags[n] & PF_ROYAL)
@@ -230,6 +268,10 @@ struct piece_description_t {
             };
             printf("\n");
          }
+         if (piece_flags[n] & PF_CAPTUREFLAG)
+            printf("   May capture the flag\n");
+         if (piece_flags[n] & PF_IRON)
+            printf("   May not be captured\n");
 
          if (royal_pieces & (1<<n)) printf("   Royal piece\n");
          if (pawn_pieces & (1<<n))  printf("   Pawn-class piece\n");
@@ -248,8 +290,6 @@ struct piece_description_t {
       }
       //if (game->board.rule_flags & RF_KING_TRAPPED)
       //   printf("Trapped kings\n");
-      //if (game->board.rule_flags & RF_KING_CORNERDRIVE)
-      //   printf("Kings can drive\n");
    }
 
 };
