@@ -248,7 +248,7 @@ inline bool zugzwang_threat()
 
 bool is_pawn_push(move_t move, side_t side)
 {
-   if (pt.pawn_pieces & (1 << get_move_piece(move))) {
+   if (pt.pawn_pieces & (1 << board.get_piece(get_move_from(move)))) {
       if (bitboard_t<kind>::board_homeland[next_side[side]].test(get_move_from(move)))
          return true;
    }
@@ -461,7 +461,7 @@ int qsearch(int alpha, int beta, int draft, int depth)
 
       /* Make sure mate wasn't caused by an illegal drop */
       if (board.check() && is_drop_move(prev_move)) {
-         int p = get_move_piece(prev_move);
+         int p = board.get_piece(get_move_to(prev_move));
          if (pt.piece_flags[p] & PF_DROPNOMATE)
             return ILLEGAL;
       }
@@ -517,7 +517,8 @@ int static_qsearch(int beta, int depth)
       cdone.set(get_move_to(move));
 
       int value = see(move);
-      //if (is_promotion_move(move) && value > 0) value += pt.piece_value[get_move_promotion_piece(move)] - pt.piece_value[get_move_piece(move)];
+      //if (is_promotion_move(move) && value > 0)
+      //   value += pt.piece_value[get_move_promotion_piece(move)] - pt.piece_value[board.get_piece(get_move_to(prev_move))];
       int score = static_score + value;
 
       if (score > best_score) {
@@ -590,7 +591,7 @@ int search(int alpha, int beta, int draft, int depth)
     * If it were, we would not get here.
     */
    if (depth > 0) {
-      if (board.fifty_counter >= 101) {
+      if (fifty_limit && board.fifty_counter >= fifty_limit) {
          if (!board.check())      /* defer the 50-move claim when in check, in case it's mate */
             return LEGALDRAW;
       }
@@ -653,6 +654,10 @@ int search(int alpha, int beta, int draft, int depth)
             return -bare_king_score;
          }
       }
+
+      if (check_limit && board.check_count[board.side_to_move] >= check_limit)
+         return (check_score == LEGALDRAW) ? check_score
+                                           : check_score + depth;
    }
 
    //if ( (depth == 0 || depth == 1 || draft > 8 || is_mate_score(beta)) && (board.rule_flags & RF_USE_DROPS)) {
@@ -672,16 +677,16 @@ int search(int alpha, int beta, int draft, int depth)
       return qsearch(alpha, beta, 0, depth);
 
    /* Test for transposition table cut-offs */
-   int hash_depth, hash_score;
-   unsigned int hash_flag;
+   int hash_depth = 0, hash_score = 0;
+   unsigned int hash_flag = 0;
    move_t hash_move = 0;
    bool have_hash = retrieve_table(transposition_table, board.hash, &hash_depth, &hash_score, &hash_flag, &hash_move);
    bool hash_ok   = hash_depth >= draft || is_mate_score(hash_score);
-   if (have_hash && hash_ok && board.fifty_counter < 80 && depth > 0) {
+   if (have_hash && hash_ok && (fifty_limit == 0 || board.fifty_counter < fifty_scale_limit) && depth > 0) {
       hash_score = score_from_hashtable(hash_score, depth);
       //positions_in_hashtable++;
       //bool exact_ok = (depth > 1) || (beta == alpha+1);
-      bool exact_ok = (beta == alpha+1);     // FIXME: use crafty-like PV table
+      bool exact_ok = (beta == alpha+1) || (hash_score <= alpha) || hash_score >= beta;     // FIXME: use crafty-like PV table
       if ((hash_flag & HASH_TYPE_EXACT) && exact_ok) {
          if (hash_score > alpha) backup_principle_variation(depth, hash_move);
 
@@ -732,7 +737,7 @@ int search(int alpha, int beta, int draft, int depth)
    if (!avoid_null) {
       assert(need_static);
       assert(!player_in_check(me));
-      int r = 2;
+      int r = 2 + draft / 4;
       int score;
       playmove(0);
       clock.nodes_searched++;
@@ -774,6 +779,8 @@ int search(int alpha, int beta, int draft, int depth)
    /* Search the first move */
    hash_flag = HASH_TYPE_UPPER;
    while ((move = movelist[depth].next_move())) {
+      /* Multi-pv mode */
+      if (multipv > 1 && depth == 0 && exclude.contains(move)) continue;
       int move_score = movelist[depth].get_move_score();
       playmove(move);
       if (player_in_check(me)) {   /* Illegal move */
@@ -808,6 +815,7 @@ int search(int alpha, int beta, int draft, int depth)
    best_score = score;
    hash_move = move;
    while (alpha < beta && (move = movelist[depth].next_move())) {
+      if (multipv > 1 && depth == 0 && exclude.contains(move)) continue;
       bool in_check = board.check();
 
       int move_score = movelist[depth].get_move_score();
@@ -909,13 +917,14 @@ no_moves:
 
       /* Make sure mate wasn't caused by an illegal drop */
       if (board.check() && is_drop_move(prev_move)) {
-         int p = get_move_piece(prev_move);
+         int p = board.get_piece(get_move_to(prev_move));
          if (pt.piece_flags[p] & PF_DROPNOMATE)
             return ILLEGAL;
       }
 
       if (board.bbc[board.side_to_move].is_empty())
-         best_score = no_piece_score + depth;
+         best_score = (no_piece_score < 0) ? no_piece_score + depth
+                                           : no_piece_score - depth;
 
       if (best_score == depth) hash_flag = HASH_TYPE_UPPER;
 
@@ -963,7 +972,7 @@ void print_principle_variation(int d = 0)
          iter(" %d.", (int)(root_move_number+moves_played)/2+1);
       movelist_t movelist;
       generate_legal_moves(&movelist);
-      iter(" %-6s", move_to_short_string(principle_variation[c][d], &movelist));
+      iter(" %-6s", move_to_short_string(principle_variation[c][d], &movelist, NULL, castle_san_ok));
       playmove(principle_variation[c][d]);
    }
    for (c=0; c<length_of_variation[d]; c++)
@@ -976,13 +985,13 @@ void print_principle_variation_xb(movelist_t *movelist = NULL)
    int root_move_number = start_move_count;
 
    if (board.side_to_move == BLACK) {
-      root_move_number += 1;
       xb(" %d. ...", (int)(root_move_number+moves_played)/2+1);
+      root_move_number += 1;
    }
    for (c=0; c<length_of_variation[0]; c++) {
       if (((root_move_number+c) & 1) == 0)
          xb(" %d.", (int)(root_move_number+moves_played)/2+1);
-      xb(" %s", move_to_short_string(principle_variation[c][0], movelist));
+      xb(" %s", move_to_short_string(principle_variation[c][0], movelist, NULL, castle_san_ok));
       playmove(principle_variation[c][0]);
    }
    for (c=0; c<length_of_variation[0]; c++)
@@ -993,7 +1002,6 @@ void print_principle_variation_xb(movelist_t *movelist = NULL)
 void print_principle_variation_uci()
 {
    int c;
-   int root_move_number = 0;
 
    for (c=0; c<length_of_variation[0]; c++) {
       uci(" %s", move_to_lan_string(principle_variation[c][0], false));
@@ -1027,8 +1035,15 @@ play_state_t get_game_end_state(movelist_t *movelist = NULL)
    if ((board.rule_flags & RF_USE_SHAKMATE) && board.check()) test_shak();
 
    /* Count a number of obvious things: repetition draw, 50 move rule */
-   if (board.fifty_counter >= 101)
-      return SEARCH_GAME_ENDED_50_MOVE;
+   if (fifty_limit && board.fifty_counter >= fifty_limit) {
+      /* Take care: we are allowed to mate on the 50th move */
+      if (board.check()) {
+         generate_legal_moves(movelist);
+         if (movelist->num_moves)
+            return SEARCH_GAME_ENDED_50_MOVE;
+      } else
+         return SEARCH_GAME_ENDED_50_MOVE;
+   }
 
    if (count_repetition() >= repeat_claim && !(board.rule_flags & RF_USE_CHASERULE))
       return SEARCH_GAME_ENDED_REPEAT;
@@ -1054,7 +1069,7 @@ play_state_t get_game_end_state(movelist_t *movelist = NULL)
        */
       if (moves_played) {
          move_t prev_move = move_list[moves_played-1];
-         int p = get_move_piece(prev_move);
+         int p = board.get_piece(get_move_to(prev_move));
          if (is_drop_move(prev_move)) {
             if (pt.piece_flags[p] & PF_DROPNOMATE)
                return SEARCH_GAME_ENDED_FORFEIT;
@@ -1096,9 +1111,138 @@ play_state_t get_game_end_state(movelist_t *movelist = NULL)
          return SEARCH_GAME_ENDED_WINBARE;
    }
 
+   if (check_limit && board.check_count[board.side_to_move] >= check_limit)
+      return SEARCH_GAME_ENDED_CHECK_COUNT;
+
    return SEARCH_OK;
 }
 
+void score_static_moves(movelist_t *movelist)
+{
+   const side_t me = board.side_to_move;
+   movelist_t omoves;
+   bitboard_t<kind> threat;
+   int threat_see = 0;
+
+   playmove(0);
+   generate_legal_moves(&omoves);
+   for (int n = 0; n<movelist->num_moves; n++) {
+      move_t move = movelist->move[n];
+      if (is_capture_move(move)) {
+         int s = see(move);
+         if (s > threat_see) {
+            threat.clear();
+            threat.set(get_move_to(move));
+            threat_see = s;
+         }
+      }
+   }
+   takeback();
+
+   int phase = 0;
+   for (side_t side = WHITE; side<NUM_SIDES; side++) {
+      for (int n=0; n<pt.num_piece_types; n++) {
+         int *perm = pt.val_perm;
+         int piece = perm[n];
+         phase += pt.phase_weight[piece] * board.holdings[piece][side];
+         phase += pt.phase_weight[piece] * (board.bbp[piece] & board.bbc[side]).popcount();
+      }
+   }
+
+   for (int n = 0; n<movelist->num_moves; n++) {
+      move_t move = movelist->move[n];
+      movelist->score[n] = 0;
+      if (is_promotion_move(move)) {
+         int s = see(move);
+         if (s >= 0)
+            movelist->score[n] += 2200 + s + pt.piece_value[get_move_promotion_piece(move)];
+         else
+            movelist->score[n] += s;
+      } else if (is_castle_move(move)) {
+         movelist->score[n] += 600;
+      } else if (is_capture_move(move)) {
+         int s = see(move);
+         if (s > 0)
+            movelist->score[n] += 2400 + s / 10;
+         else
+            movelist->score[n] += s;
+      } else if (is_drop_move(move)) {
+         int s = see(move);
+         if (s >= 0) {
+            if (!(board.bbc[me] & board.royal & bitboard_t<kind>::neighbour_board[get_move_to(move)]).is_empty())
+               movelist->score[n] += 1200;
+            else if (bitboard_t<kind>::board_homeland[next_side[me]].test(get_move_to(move)))
+               movelist->score[n] += 1100;
+            movelist->score[n] += s / 10;
+         } else {
+            movelist->score[n] += s;
+         }
+      } else {
+         int s = see(move);
+         movelist->score[n] += s;
+      }
+
+      int piece = get_move_piece(move);
+      int pst = centre_table[get_move_to(move)] - centre_table[get_move_from(move)];
+      if (pt.royal_pieces & (1<<piece)) {
+         movelist->score[n] += (-pst*4*phase + pst*(pt.phase_scale - phase)) / pt.phase_scale;
+      } else {
+         movelist->score[n] += pst;
+      }
+
+      /* Penalise moves that just unmake the last move */
+      if (moves_played >= 2) {
+         move_t prev_move = move_list[moves_played-2];
+
+         if (get_move_from(move) == get_move_to(prev_move)) {
+            movelist->score[n] -= 10;
+            if (get_move_to(move) == get_move_from(prev_move))
+               movelist->score[n] -= 100;
+         }
+      }
+
+      /* Push passed pawns */
+      if (pt.pawn_pieces & (1<<piece)) {
+         pawn_structure_t<kind> ps;
+         calculate_pawn_structure(&ps);
+
+         if (ps.passed.test(get_move_from(move)))
+            movelist->score[n] += 50;
+      }
+
+      /* TODO:
+       *  - Close with defending king in end game.
+       *  - Some sense of game phase (for pushing pawns)
+       *  - Some sort of threat detection: double attacks, attacking pinned
+       *    pieces, and the like.
+       */
+
+      if (threat_see && !threat.test(get_move_from(move)))
+         movelist->score[n] -= threat_see;
+
+
+      if (is_gate_move(move))
+         movelist->score[n] += 200;
+
+      playmove(move);
+      generate_legal_moves(&omoves);
+      if (player_in_check(next_side[me])) {
+         movelist->score[n] += 100;
+         if (omoves.num_moves == 0)
+            movelist->score[n] += 2000;
+      } else {
+         if (omoves.num_moves == 0)
+            movelist->score[n] = -50;
+      }
+      takeback();
+
+      movelist->score[n] *= 8;
+      movelist->score[n] += genrandui() & 0xf;
+
+   }
+   //movelist->show();
+}
+
 play_state_t think(int max_depth)
 {
    play_state_t state;
@@ -1128,15 +1272,39 @@ play_state_t think(int max_depth)
    for (side_t side = WHITE; side<=BLACK; side++)
       if ((board.bbc[side] & ~board.royal).is_empty()) mate_search = true;
 
+   /* Level of play */
+   switch (level) {
+      /* Pure random mover */
+      case LEVEL_RANDOM:
+         playmove(movelist.move[int(genrandf()*movelist.num_moves)]);
+         return SEARCH_OK;
+         break;
+
+      case LEVEL_STATIC:
+         score_static_moves(&movelist);
+         //movelist.show();
+         movelist.rewind();
+         playmove(movelist.next_move());
+         return SEARCH_OK;
+         break;
+
+      /* Normal alpha/beta search */
+      default:
+         break;
+   }
+
    /* Prepare the transpostion table for a new search iteration:
     * Resets the write count and increases the generation counter.
     */
-   prepare_hashtable_search(transposition_table);
+   if (!analysing)
+      prepare_hashtable_search(transposition_table);
 
    int hash_depth, hash_score;
    uint32_t hash_flag;
    retrieve_table(transposition_table, board.hash, &hash_depth, &hash_score, &hash_flag, &best_move[0]);
 
+   xb("# Begin iterative deepening loop for position \"%s\"\n", make_fen_string());
+
    /* Iterative deepening loop */
    int e = board.check();
    clock.extra_time = clock.time_left;
@@ -1161,77 +1329,83 @@ play_state_t think(int max_depth)
    for (depth=2; depth<=max_depth; depth++) {
       scale_history();
       movelist.rewind();
+      exclude.clear();
 
-      for (int window = 10; window < 2*LEGALWIN; window *= 2) {
-         if (depth <= 2) window = LEGALWIN / (depth - 1);
-         if (mate_search && !is_mate_score(score)) window = LEGALWIN;
-         int alpha = std::max(score - window, -LEGALWIN);
-         int beta  = std::min(score + window,  LEGALWIN);
+      for (int pv=0; pv<multipv; pv++) {
+         if (pv > movelist.num_moves || exclude.num_moves >= movelist.num_moves) break;
+         for (int window = 10; window < 2*LEGALWIN; window *= 2) {
+            if (depth <= 2) window = LEGALWIN / (depth - 1);
+            if (mate_search && !is_mate_score(score)) window = LEGALWIN;
+            int alpha = std::max(score - window, -LEGALWIN);
+            int beta  = std::min(score + window,  LEGALWIN);
 
-         int new_score = search(alpha, beta, depth + e, 0);
-         clock.extra_time = 0;
+            int new_score = search(alpha, beta, depth + e, 0);
+            clock.extra_time = 0;
 
-         if (abort_search) break;
-         score = new_score;
+            if (abort_search) break;
+            score = new_score;
 
-         if (score > alpha && score < beta) {
-            move = best_move[0];
-            break;
-         }
-
-         /* Report a fail-low */
-         if (score < alpha && move) {
-            move_t move = best_move[0];
-            if (show_fail_low) {
-               iter("% 3d.  %6.2f   %9d  %+2.2f  ", depth, (get_timer()-start_time)/1000000.0, (int)clock.nodes_searched, score/100.0);
-               iter(" %d.", (int)(start_move_count + moves_played)/2+1);
-               if (board.side_to_move) iter(" ...");
-               iter(" %s?", move_to_short_string(move, &movelist));
-               iter("\n");
-
-               xb("% 3d % 5d %6d %9d ", depth, score, (int)(get_timer()-start_time)/10000, (int)clock.nodes_searched);
-               xb(" %d.", (int)(start_move_count + moves_played)/2+1);
-               if (board.side_to_move) xb(" ...");
-               xb(" %s?", move_to_short_string(move, &movelist));
-               xb("\n");
+            if (score > alpha && score < beta) {
+               move = best_move[0];
+               break;
             }
-            clock.extra_time = (9*clock.time_left)/10;
-         }
 
+            /* Report a fail-low */
+            if (score < alpha && move) {
+               move_t move = best_move[0];
+               if (show_fail_low) {
+                  iter("% 3d.  %6.2f   %9d  %+2.2f  ", depth, (get_timer()-start_time)/1000000.0, (int)clock.nodes_searched, score/100.0);
+                  iter(" %d.", (int)(start_move_count + moves_played)/2+1);
+                  if (board.side_to_move) iter(" ...");
+                  iter(" %s?", move_to_short_string(move, &movelist, NULL, castle_san_ok));
+                  iter("\n");
 
-         /* Report a fail-high */
-         if (score >= beta) {
-            move = best_move[0];
-            if (show_fail_high) {
-               iter("% 3d.  %6.2f   %9d  %+2.2f  ", depth, (get_timer()-start_time)/1000000.0, (int)clock.nodes_searched, score/100.0);
-               iter(" %d.", (int)(start_move_count + moves_played)/2+1);
-               if (board.side_to_move) iter(" ...");
-               iter(" %s!", move_to_short_string(move, &movelist));
-               iter("\n");
+                  xb("% 3d % 5d %6d %9d ", depth, score, (int)(get_timer()-start_time)/10000, (int)clock.nodes_searched);
+                  xb(" %d.", (int)(start_move_count + moves_played)/2+1);
+                  if (board.side_to_move) xb(" ...");
+                  xb(" %s?", move_to_short_string(move, &movelist, NULL, castle_san_ok));
+                  xb("\n");
+               }
+               clock.extra_time = (9*clock.time_left)/10;
+            }
 
-               xb("% 3d % 5d %6d %9d ", depth, score, (int)(get_timer()-start_time)/10000, (int)clock.nodes_searched);
-               xb(" %d.", (int)(start_move_count + moves_played)/2+1);
-               if (board.side_to_move) xb(" ...");
-               xb(" %s!", move_to_short_string(move, &movelist));
-               xb("\n");
+
+            /* Report a fail-high */
+            if (score >= beta) {
+               move = best_move[0];
+               if (show_fail_high) {
+                  iter("% 3d.  %6.2f   %9d  %+2.2f  ", depth, (get_timer()-start_time)/1000000.0, (int)clock.nodes_searched, score/100.0);
+                  iter(" %d.", (int)(start_move_count + moves_played)/2+1);
+                  if (board.side_to_move) iter(" ...");
+                  iter(" %s!", move_to_short_string(move, &movelist, NULL, castle_san_ok));
+                  iter("\n");
+
+                  xb("% 3d % 5d %6d %9d ", depth, score, (int)(get_timer()-start_time)/10000, (int)clock.nodes_searched);
+                  xb(" %d.", (int)(start_move_count + moves_played)/2+1);
+                  if (board.side_to_move) xb(" ...");
+                  xb(" %s!", move_to_short_string(move, &movelist, NULL, castle_san_ok));
+                  xb("\n");
+               }
+               clock.extra_time = (5*clock.time_left)/10;
             }
-            clock.extra_time = (5*clock.time_left)/10;
          }
+         iter("% 3d.  %6.2f   %9d  %+2.2f  ", depth, (get_timer() - start_time)/1000000.0, (int)clock.nodes_searched, score/100.0);
+         print_principle_variation();
+         iter("\n");
+
+         xb("% 3d % 5d %6d %9d ", depth, score, (int)(get_timer()-start_time)/10000, (int)clock.nodes_searched);
+         print_principle_variation_xb(&movelist);
+         xb("\n");
+
+         uci("info depth %d score cp %d nodes %d time %d hashfull %d pv",
+               depth, score, (int)clock.nodes_searched, (int)peek_timer(&clock), (int)std::min(1000, (int)(transposition_table->write_count*1000/transposition_table->number_of_elements)));
+         print_principle_variation_uci();
+         if (multipv>1) { uci(" multipv %d", pv+1); }
+         uci("\n");
+
+         store_principle_variation(score, depth);
+         exclude.push(principle_variation[0][0]);
       }
-      iter("% 3d.  %6.2f   %9d  %+2.2f  ", depth, (get_timer() - start_time)/1000000.0, (int)clock.nodes_searched, score/100.0);
-      print_principle_variation();
-      iter("\n");
-
-      xb("% 3d % 5d %6d %9d ", depth, score, (int)(get_timer()-start_time)/10000, (int)clock.nodes_searched);
-      print_principle_variation_xb(&movelist);
-      xb("\n");
-
-      uci("info depth %d score cp %d nodes %llu time %llu hashfull %d pv",
-            depth, score, (unsigned long long)clock.nodes_searched, (unsigned long long)peek_timer(&clock), (int)std::min(1000, (int)(transposition_table->write_count*1000/transposition_table->number_of_elements)));
-      print_principle_variation_uci();
-      uci("\n");
-
-      store_principle_variation(score, depth);
 
       if (is_mate_score(score) && (LEGALWIN - abs(score)) == length_of_variation[0] && move && !analysing) break;
       if (abort_search) break;
@@ -1247,6 +1421,9 @@ play_state_t think(int max_depth)
    while (analysing && !abort_search)
       check_clock();
 
+   while (pondering && !abort_search)
+      check_clock();
+
    if (score < resign_threshold)
       resign_count++;
    else
@@ -1300,6 +1477,9 @@ bool ponder()
    if (think(MAX_SEARCH_DEPTH) == SEARCH_OK)
       takeback();
 
+   while (pondering && !abort_search)
+      check_clock();
+
    takeback();
    pondering = false;
 
