@@ -41,11 +41,44 @@ const char *move_to_lan_string(move_t move, bool castle_san, bool castle_kxr, ch
          p = "+";
    }
 
-   if (get_move_from(move) == get_move_to(move)) {
+   if (get_move_from(move) == get_move_to(move) && !is_capture_move(move) && !is_castle_move(move) && !is_drop_move(move)) {
       snprintf(s, sizeof static_buffer, "@@@@");
       return s;
    }
 
+   if (is_double_capture_move(move)) {
+      int from = get_move_from(move);
+      int to = get_move_to(move);
+      int first = 1;
+      int c1, c2;
+      uint16_t p = 0;
+   
+      if (get_move_swaps(move))
+         first = 0;
+
+      p = get_move_pickup(move, first);
+      c1 = decode_pickup_square(p);
+      p = get_move_pickup(move, first+1);
+      c2 = decode_pickup_square(p);
+      if (c2 == to) {
+         snprintf(s, 256, "%s%s,%s%s", square_names[from], square_names[c1], square_names[c1], square_names[to]);
+      } else {
+         snprintf(s, 256, "%s%s,%s%s,%s%s", square_names[from], square_names[c1], square_names[c1], square_names[c2], square_names[c2], square_names[to]);
+      }
+      return s;
+   }
+
+   if (is_capture_move(move) && get_move_capture_square(move) != get_move_to(move)) {
+      int from = get_move_from(move);
+      int to = get_move_to(move);
+      int cap = get_move_capture_square(move);
+      char fp = piece_symbol_string[get_move_piece(move)];
+      if (fp != ' ') {
+         snprintf(s, 256, "%s%s,%s%s", square_names[from], square_names[cap], square_names[cap], square_names[to]);
+         return s;
+      }
+   }
+
    if (is_castle_move(move)) {
       int from = get_move_from(move);
       int to = get_move_to(move);
@@ -58,6 +91,11 @@ const char *move_to_lan_string(move_t move, bool castle_san, bool castle_kxr, ch
          snprintf(s, sizeof static_buffer, "%s%s%c", square_names[from], square_names[get_castle_move_from2(move)], tolower(*p));
       } else {
          snprintf(s, sizeof static_buffer, "%s%s%c", square_names[from], square_names[to], tolower(*p));
+         if (abs(from-to) == 1) {
+            int from2 = get_castle_move_from2(move);
+            int to2 = get_castle_move_to2(move);
+            snprintf(s, sizeof static_buffer, "%s%s,%s%s", square_names[from], square_names[to], square_names[from2], square_names[to2]);
+         }
       }
       if (is_gate_move(move)) {
          if (get_move_drop_square(move) != from) {
@@ -76,14 +114,15 @@ const char *move_to_lan_string(move_t move, bool castle_san, bool castle_kxr, ch
          p = piece_symbol_string + get_move_piece(move);
          snprintf(s, sizeof static_buffer, "%c@%s%c", piece_drop_string[piece], square_names[get_move_to(move)], tolower(*p));
       }
+      return s;
    } else if (is_pickup_move(move))
       //snprintf(s, sizeof static_buffer, "%c^%s", piece_drop_string[get_move_piece(move)], square_names[get_move_from(move)]);
       snprintf(s, sizeof static_buffer, "@@%s", square_names[get_move_from(move)]);
    else
       snprintf(s, sizeof static_buffer, "%s%s%c", square_names[get_move_from(move)], square_names[get_move_to(move)], tolower(*p));
 
-   if (is_gate_move(move))
-      snprintf(s + strlen(s), sizeof static_buffer-strlen(s), "%c", tolower(*p));
+   //if (is_gate_move(move))
+   //   snprintf(s + strlen(s), sizeof static_buffer-strlen(s), "%c", tolower(*p));
 
    return s;
 }
@@ -131,6 +170,40 @@ const char *move_to_string(move_t move, char *buffer)
       return s;
    }
 
+   if (is_double_capture_move(move)) {
+      int from = get_move_from(move);
+      int to = get_move_to(move);
+      char fp = piece_symbol_string[get_move_piece(move)];
+      int first = 1;
+      int c1, c2;
+      uint16_t p = 0;
+   
+      if (get_move_swaps(move))
+         first = 0;
+
+      p = get_move_pickup(move, first);
+      c1 = decode_pickup_square(p);
+      p = get_move_pickup(move, first+1);
+      c2 = decode_pickup_square(p);
+      if (c2 == to) {
+         snprintf(s, 256, "%c%sx%sx%s", fp, square_names[from], square_names[c1], square_names[to]);
+      } else {
+         snprintf(s, 256, "%c%sx%sx%s-%s", fp, square_names[from], square_names[c1], square_names[c2], square_names[to]);
+      }
+      return s;
+   }
+
+   if (is_capture_move(move) && get_move_capture_square(move) != get_move_to(move)) {
+      int from = get_move_from(move);
+      int to = get_move_to(move);
+      int cap = get_move_capture_square(move);
+      char fp = piece_symbol_string[get_move_piece(move)];
+      if (fp != ' ') {
+         snprintf(s, 256, "%c%sx%s-%s", fp, square_names[from], square_names[cap], square_names[to]);
+         return s;
+      }
+   }
+
    /* Normal move or capture */
    if (is_capture_move(move))
       dash = 'x';
